#!/usr/bin/env python

# We need sockets, threads and the hashlib
import socket, threading, hashlib

# The main class of the server
class server:

	# The IP address
	ip='0.0.0.0'

	# The port
	port=54321

	# The number of max connections
	maxconn=0

	# The main socket
	mainsocket=None

	# Tells if the main socket is ok
	mainready=False

	# The client threads
	clients={}

	# Initialize the main socket
	def createMain(self):
		# Create the socket
		try:
			mainsocket=socket.socket(socket.AF_INET, socket.SOCK_STREAM)
			mainsocket.bind((self.ip, self.port))
		except:
			self.log('Cannot bind the main socket.')
			return False

		# Listen on it
		try:
			mainsocket.listen(self.maxconn)
		except:
			self.log('Cannot listen on the socket.')
			return False

		# Share it with the class
		self.mainsocket=mainsocket
		self.mainready=True

	# Start to listen on the main socket
	def accept(self):
		if not self.mainready:
			raise Exception()
			return False

		# Wait for a connection
		conn, addr = self.mainsocket.accept()

		# Build the id for the connection
		cid=addr[0] + ':' + `addr[1]`
		m=hashlib.md5()
		m.update(cid)
		cid=m.hexdigest()
		self.log('Connected by ' + cid)

		# Init the client
		self.clients[cid]=client(cid, conn, self)
		self.clients[cid].start()

	# Close the main socket
	def closeMain(self):
		if not self.mainready:
			return True

		try:
			self.mainsocket.close()
			return True
		except:
			self.log('Cannot close main socket')
			return False

	# Close the clients
	def closeClients(self):
		# Send the closing signal to clients
		for i in self.clients:
			# Don't destroy clients or the for will complain
			self.clients[i].close(True);

		# Reset the clients dict
		self.clients={}

	# Log the errors
	def log(self, msg):
		print(msg)

# The client class
class client(threading.Thread):

	# The id of the client
	id=''

	# The connection
	socket=''

	# Can receive?
	receive=1

	# The server instance
	server=None

	# The constructor
	def __init__(self, id, socket, server):
		threading.Thread.__init__(self)
		self.id=id
		self.socket=socket
		self.server=server

	# The running method
	def run(self):
		socket=self.socket

		# Let's enter in the infinite loop
		while self.receive:
			data=socket.recv(1024)
			if not data: break
			if data[0:4]=='exit': break

			# Call the callback
			self.received(data)

		# When exiting the while, close the socket
		self.close()

	# The handler for received data
	def received(self, data):
		socket=self.socket

		# Echo to other clients the received data
		for i in self.server.clients:
			if i==self.id:
				continue
			client=self.server.clients[i]
			client.send(data)

	# Send a message to the client
	def send(self, data):
		self.socket.send(data)

	# Destory the clients
	#
	# The param nodestroy is useful when closing
	# the clients with an itineration.
	def close(self, nodestroy=False):
		self.log('Disconnetting')
		self.receive=0
		self.socket.close();

		# Destroy from the server this client?
		if not nodestroy:
			del self.server.clients[self.id]

	# Log. By default it uses the server class
	def log(self, msg):
		self.server.log('Client '+self.id+': '+msg)

# Create the instance
s=server()
s.createMain()

# Listen for clients
while 1:
	try:
		s.accept()
	except KeyboardInterrupt:
		print('KeyboardInterrupt received. Closing the server.')
		break
	except:
		break

# Close the program
s.closeClients()
s.closeMain()